/*
 * Copyright (c) 2017, Oracle and/or its affiliates. All rights reserved.
* ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
*/

package com.sun.ts.tests.jpa.jpa22.generators.tablegenerators;

import com.sun.javatest.Status;
import com.sun.ts.lib.util.TestUtil;
import com.sun.ts.tests.jpa.common.PMClientBase;

import java.util.Properties;

public class Client extends PMClientBase {
	private static final long serialVersionUID = 22L;
	private DataTypes d0;

    public Client() {
    }

    public static void main(String[] args) {
        Client theTests = new Client();
        Status s = theTests.run(args, System.out, System.err);
        s.exit();
    }

    public void setup(String[] args, Properties p) throws Fault {
        TestUtil.logTrace("setup");
        try {

            super.setup(args, p);
            removeTestData();
            createTestData();
        } catch (Exception e) {
            TestUtil.logErr("Exception: ", e);
            throw new Fault("Setup failed:", e);
        }
    }

    /*
     * @testName: generatorOnEntityTest
     * @assertion_ids: PERSISTENCE:JAVADOC:3489;
     * @test_Strategy:  use a generator specified on an entity
     */

    public void generatorOnEntityTest() throws Fault {

        boolean pass = false;

            try {
                getEntityTransaction().begin();
                int id = d0.getId();
                TestUtil.logTrace("find id: " + id);
                DataTypes d = getEntityManager().find(DataTypes.class, id);
                if (d != null) {
                    if (d.getStringData().equals(d0.getStringData())) {
                        pass = true;
                    }

                    getEntityTransaction().commit();
                } else {
                    TestUtil.logErr("EntityManager.find returned null result");
                }
            } catch (Exception e) {
                TestUtil.logErr("Unexpected exception occurred", e);
            }

        if (!pass)
            throw new Fault("generatorOnEntityTest failed");
    }
    // Methods used for Tests

    public void createTestData() {
        try {
            getEntityTransaction().begin();

            d0 = new DataTypes();
            d0.setStringData("testData");
            TestUtil.logTrace("DataType:" + d0.toString());
            getEntityManager().persist(d0);

            getEntityManager().flush();
            getEntityTransaction().commit();

        } catch (Exception e) {
            TestUtil.logErr("Unexpected exception occurred", e);
        }
    }
     public void cleanup() throws Fault {
        TestUtil.logTrace("Cleanup data");
        removeTestData();
        TestUtil.logTrace("cleanup complete, calling super.cleanup");
        super.cleanup();
    }

    private void removeTestData() {
        TestUtil.logTrace("removeTestData");
        if (getEntityTransaction().isActive()) {
            getEntityTransaction().rollback();
        }
        try {
            getEntityTransaction().begin();
            getEntityManager().createNativeQuery("DELETE FROM DATATYPES").executeUpdate();
            getEntityTransaction().commit();
        } catch (Exception e) {
            TestUtil.logErr("Exception encountered while removing entities:", e);
        } finally {
            try {
                if (getEntityTransaction().isActive()) {
                    getEntityTransaction().rollback();
                }
            } catch (Exception re) {
                TestUtil.logErr("Unexpected Exception in removeTestData:", re);
            }
        }
    }

}

